"""Abstractions for the GTK event loop."""

__all__ = [
    'BlockingLoop',
    'GtkLoop',
    ]

import gobject
import gtk


class GtkLoop(object):
    """Wrapper around the GTK+ event loop."""

    def is_running(self):
        return gtk.main_level() > 0

    def start(self):
        gtk.main()

    def stop(self):
        gtk.main_quit()

    def run_when_idle(self, function):
        gobject.idle_add(function)


class BlockingLoop(object):
    """A 'blocking loop'. Not actually an event loop at all.

    Simulates the loop interface without actually creating a blocking loop.
    The biggest difference is that with "real" loops, `start` will block.
    """

    def __init__(self):
        self._running = False
        self._pending_functions = []

    def _run_pending_functions(self):
        for function in self._pending_functions:
            while self.is_running() and function():
                pass

    def is_running(self):
        return self._running

    def start(self):
        self._running = True
        self._run_pending_functions()

    def stop(self):
        self._running = False

    def run_when_idle(self, function):
        self._pending_functions.append(function)
        if self.is_running():
            self._run_pending_functions()
