__all__ = [
    'TestRunner',
    'main',
    'make_runner',
    ]

import gtk

from tribunal import providers
from tribunal.eventloop import GtkLoop
from tribunal.view import ExtendedResult, load_glade, TribunalWindow


class TestRunner(object):

    def __init__(self, test_provider, view, event_loop=None):
        if event_loop is None:
            event_loop = GtkLoop()
        self._event_loop = event_loop
        self._test_provider = test_provider
        self._view = self.view = view
        self._stop = None

    def get_signal_handlers(self):
        prefix = 'event_'
        handlers = {}
        for name in self.__class__.__dict__.keys():
            if not name.startswith(prefix):
                continue
            method = getattr(self, name)
            name = name[len(prefix):]
            handlers[name] = method
        return handlers

    def show_all(self):
        self._view.show_all()

    def load_tests(self, name):
        tests = list(self._test_provider.find_tests(name))
        self._view.tests_loaded(tests)

    def event_test_activated(self, view, path, column):
        self.set_name(self._view.tree.get_test_id(path))
        self.run()

    def event_run_clicked(self, button):
        self.run()

    def event_window_destroyed(self, widget):
        self._event_loop.stop()

    def event_load_tests(self, button):
        name = self.get_name()
        self.load_tests(name)

    def event_key_pressed(self, widget, event):
        # XXX -- yuck
        key = gtk.gdk.keyval_name(event.keyval).lower()
        if event.state & gtk.gdk.CONTROL_MASK:
            if key == 'q':
                self._event_loop.stop()
                return True
        else:
            if key == 'f5':
                self.run()
                return True
            if key == 'escape':
                self.stop()
                return True

    def set_name(self, name):
        self._view.testNameField.set_text(name)

    def get_name(self):
        return self._view.testNameField.get_text()

    def stop(self):
        if self._stop:
            self._stop()

    def _run_tests(self, name, result):
        for test in self._test_provider.run_tests(name, result):
            yield True
        self.suite_finished(result)
        yield False

    def run(self):
        name = self.get_name()
        self.load_tests(name)
        result = ExtendedResult(self._view)
        self._stop = result.stop
        self._event_loop.run_when_idle(self._run_tests(name, result).next)

    def suite_finished(self, result):
        result.done()
        self._stop = None


def make_runner(provider, event_loop):
    glade_xml = load_glade()
    window = TribunalWindow(glade_xml)
    runner = TestRunner(provider, window, event_loop)
    glade_xml.signal_autoconnect(runner.get_signal_handlers())
    return runner


def main(args):
    """Launch the Tribunal test runner.

    :param args: The arguments from the command line. Only the first real
        argument is used (i.e. args[1]).
    """
    event_loop = GtkLoop()
    runner = make_runner(providers.SubunitServer(), event_loop)
    runner.show_all()
    if len(args) > 1:
        runner.set_name(args[1])
        runner.run()
    event_loop.start()

