import gtk


class ProgressBar(gtk.DrawingArea):
    """Custom progress bar for running tests.

    This progress bar gradually fills an area with green until it encounters
    its first failure.
    """

    failure = gtk.gdk.color_parse('#FF0000')
    success = gtk.gdk.color_parse('#00FF00')

    def __init__(self):
        gtk.DrawingArea.__init__(self)
        self._steps = 1
        self._stepsTaken = 0
        self._status = self.success
        self.connect('realize', self.on_realize)
        self.connect('expose-event', self.on_expose)

    def on_realize(self, widget):
        self.gc = self.window.new_gc()
        self.gc.set_fill(gtk.gdk.SOLID)

    def on_expose(self, widget, event):
        drawable = self.window
        width, height = drawable.get_size()
        self.gc.set_rgb_fg_color(self._status)
        x2 = int(width * self.get_fraction())
        y2 = int(height)
        drawable.draw_rectangle(self.gc, True, 0, 0, x2, y2)

    def step(self, success):
        """Increment the progress bar.

        :param success: If success is True, keep using the success color.
            Otherwise, switch the entire bar to the failure color (usually
            red).
        """
        if not success:
            self._status = self.failure
        self._stepsTaken += 1
        self.queue_draw()

    def reset(self, steps):
        """Reset the progress bar to 'no progress' and to expect a certain
        number of steps.

        :param steps: The number of steps to expect. This is generally the
            number of tests.
        """
        self._steps = steps
        self._stepsTaken = 0
        self._status = self.success
        self.queue_draw()

    def get_fraction(self):
        """Get the current progress expressed as a number between 0 and 1."""
        if self._steps == 0:
            return 0
        return float(self._stepsTaken) / float(self._steps)

    def get_color(self):
        """Return the current color of the bar."""
        return self._status

    def startTest(self, test):
        self._successful = True

    def stopTest(self, test):
        """A test has finished, update the progress bar accordingly."""
        self.step(self._successful)

    def addFailure(self, test, failure):
        self._successful = False

    def addError(self, test, error):
        self._successful = False

    def addSuccess(self, test):
        """Do nothing."""

    def done(self):
        """Do nothing."""
