"""Things that provide ways to run tests."""

__all__ = [
    'InProcessServer',
    'SubunitClient',
    'SubunitServer',
    ]

import os
import subprocess
import sys

from zope.interface import implements

from twisted.trial.runner import TestLoader

from testtools import (
    clone_test_with_new_id, iterate_tests, MultiTestResult, TestResult)

from tribunal.interfaces import ITestServer


class InProcessServer:
    """Crappy test provider."""

    implements(ITestServer)

    def __init__(self):
        self.loader = TestLoader()

    def find_tests(self, name):
        tests = self.loader.loadByName(name, recurse=True)
        for test in iterate_tests(tests):
            yield test.id()

    def run_tests(self, name, result):
        tests = self.loader.loadByName(name, recurse=True)
        for test in iterate_tests(tests):
            test.run(result)
            yield test


class SubunitClient:
    """XXX

    Run this in a sub-process and have the SubunitServer interpret the
    results.
    """

    def __init__(self, stream):
        self._stream = stream
        self._loader = TestLoader()

    def find_tests(self, name):
        import subunit
        _test_result = subunit.TestProtocolClient(self._stream)
        tests = self._loader.loadByName(name, recurse=True)
        for test in iterate_tests(tests):
            _test_result.startTest(test)
            _test_result.addSuccess(test)

    def run_tests(self, name):
        import subunit
        _test_result = subunit.TestProtocolClient(self._stream)
        tests = self._loader.loadByName(name, recurse=True)
        tests.run(_test_result)


class _LastTestStorer(TestResult):

    last_test = None

    def stopTest(self, test):
        self.last_test = test


class _SubunitResult(TestResult):
    """TestResult that removes subunit test names from test IDs."""

    def __init__(self, result):
        self._result = result

    def _get_new_id(self, test):
        return '.'.join(test.id().split('.')[2:])

    def addError(self, test, result):
        self._result.addError(
            clone_test_with_new_id(test, self._get_new_id(test)), result)

    def addFailure(self, test, result):
        self._result.addFailure(
            clone_test_with_new_id(test, self._get_new_id(test)), result)

    def addSuccess(self, test):
        self._result.addSuccess(
            clone_test_with_new_id(test, self._get_new_id(test)))

    def startTest(self, test):
        self._result.startTest(
            clone_test_with_new_id(test, self._get_new_id(test)))

    def stopTest(self, test):
        self._result.stopTest(
            clone_test_with_new_id(test, self._get_new_id(test)))

    def done(self):
        self._result.done()


class SubunitServer:

    def __init__(self):
        bin_dir = os.path.join(
            os.path.dirname(os.path.dirname(__file__)), 'bin')
        self._runner = os.path.join(bin_dir, 'subunit-runner')
        self._lister = os.path.join(bin_dir, 'subunit-lister')

    def _run_protocol(self, pipe, test_result):
        import subunit
        last_test_storer = _LastTestStorer()
        result = MultiTestResult(
            _SubunitResult(test_result), last_test_storer)
        protocol = subunit.TestProtocolServer(result)
        for line in pipe.readlines():
            protocol.lineReceived(line)
            if last_test_storer.last_test is not None:
                yield last_test_storer.last_test
                last_test_storer.last_test = None
        protocol.lostConnection()

    def _get_test_id(self, test):
        test_segments = test.id().split('.')[2:]
        if len(test_segments) > 0:
            return '.'.join(test_segments)
        else:
            return None

    def run_tests(self, name, result):
        process = subprocess.Popen(
            [sys.executable, self._runner, name], stdout=subprocess.PIPE)
        for test in self._run_protocol(process.stdout, result):
            test_id = self._get_test_id(test)
            if test_id is not None:
                yield test_id

    def find_tests(self, name):
        process = subprocess.Popen(
            [sys.executable, self._lister, name], stdout=subprocess.PIPE)
        result = TestResult()
        for test in self._run_protocol(process.stdout, result):
            test_id = self._get_test_id(test)
            if test_id is not None:
                yield test_id
