__all__ = [
    'get_header',
    'get_footer',
    'get_stacks',
    'parse_footer',
    'parse_stack',
    'Results',
    ]

import gtk
from pango import UNDERLINE_SINGLE, WEIGHT_BOLD
import re
import traceback


class LinkBuffer(object):

    HAND_CURSOR = gtk.gdk.Cursor(gtk.gdk.HAND2)
    REGULAR_CURSOR = gtk.gdk.Cursor(gtk.gdk.XTERM)

    def __init__(self, view):
        self._view = view
        self._buffer = view.get_buffer()
        self._hovering_over_link = False
        view.connect("key-press-event", self.event_key_press)
        view.connect("event-after", self.event_after)
        view.connect("motion-notify-event", self.event_motion_notify)
        view.connect("visibility-notify-event", self.event_visibility_notify)

    def _translate_coords(self, x, y):
        return self._view.window_to_buffer_coords(
            gtk.TEXT_WINDOW_WIDGET, x, y)

    def _get_event_coords(self, event):
        return self._translate_coords(int(event.x), int(event.y))

    def _get_uri_at_iter(self, iter):
        tags = iter.get_tags()
        for tag in tags:
            uri = tag.get_data('uri')
            if uri:
                return uri

    def _update_cursor(self, window, x, y):
        """Update the cursor depending on whether we're over a link.

        Looks at all tags covering the position (x, y) in the text view, and
        if one of them is a link, change the cursor to the "hands" cursor
        typically used by web browsers.
        """
        iter = self._view.get_iter_at_location(x, y)
        hovering = bool(self._get_uri_at_iter(iter))
        if hovering == self._hovering_over_link:
            # No change.
            return
        self._hovering_over_link = hovering
        if hovering:
            window.set_cursor(self.HAND_CURSOR)
        else:
            window.set_cursor(self.REGULAR_CURSOR)

    def _follow_if_link(self, iter):
        uri = self._get_uri_at_iter(iter)
        if uri:
            self.link_activated(uri)

    def event_key_press(self, text_view, event):
        # Links can be activated by pressing Enter.
        if (event.keyval == gtk.keysyms.Return or
            event.keyval == gtk.keysyms.KP_Enter):
            iter = self._buffer.get_iter_at_mark(self._buffer.get_insert())
            self._follow_if_link(iter)
        return False

    def event_after(self, text_view, event):
        # Links can also be activated by clicking.
        if event.type != gtk.gdk.BUTTON_RELEASE:
            return False
        if event.button != 1:
            return False

        # we shouldn't follow a link if the user has selected something
        try:
            start, end = self._buffer.get_selection_bounds()
        except ValueError:
            # If there is nothing selected, try to follow the link.
            pass
        else:
            if start.get_offset() != end.get_offset():
                return False

        x, y = self._get_event_coords(event)
        iter = self._view.get_iter_at_location(x, y)
        self._follow_if_link(iter)
        return False

    def event_motion_notify(self, text_view, event):
        # Update the cursor image if the pointer moved.
        x, y = self._get_event_coords(event)
        self._update_cursor(event.window, x, y)
        return False

    def event_visibility_notify(self, text_view, event):
        # Also update the cursor image if the window becomes visible
        # (e.g. when a window covering it got iconified).
        wx, wy, mod = self._view.window.get_pointer()
        bx, by = self._translate_coords(wx, wy)
        self._update_cursor(event.window, bx, by)
        return False

    def append_link(self, uri, text):
        end = self._buffer.get_end_iter()
        self.insert_link(end, uri, text)

    def append_markup(self, text, *tags):
        end = self._buffer.get_end_iter()
        self._buffer.insert_with_tags_by_name(end, text, *tags)

    def clear(self):
        """Clear the buffer."""
        self._buffer.set_text('')

    def create_tag(self, tag_name, **properties):
        return self._buffer.create_tag(tag_name, **properties)

    def create_tags(self, tags_dict):
        for name, properties in tags_dict.iteritems():
            self.create_tag(name, **properties)

    def insert_link(self, iter, uri, text):
        """Insert a link to `uri` in the buffer."""
        tag = self._buffer.create_tag(
            None, foreground="blue", underline=UNDERLINE_SINGLE)
        tag.set_data("uri", uri)
        self._buffer.insert_with_tags(iter, text, tag)

    def link_activated(self, uri):
        """Called when a link is clicked."""
        print "CLICKED: %s" % (uri,)


class Results(object):

    def __init__(self, view):
        self._view = view
        self._link_buffer = LinkBuffer(view)
        tags = {
            'test-id': dict(weight=WEIGHT_BOLD),
            'header': dict(),
            'function-name': dict(family='monospace'),
            'text': dict(family='monospace'),
            'error-class': dict(weight=WEIGHT_BOLD),
            'error-message': dict(family='monospace'),
            }
        self._link_buffer.create_tags(tags)

    def _append_test_id(self, test_id):
        self._link_buffer.append_markup(test_id, 'test-id')
        self._link_buffer.append_markup('\n')

    def _append_header(self, header):
        self._link_buffer.append_markup(header, 'header')
        self._link_buffer.append_markup('\n')

    def _append_stack(self, filename, line_no, function_name, text):
        self._link_buffer.append_markup('  File "')
        uri = 'file://%s#%s' % (filename, line_no)
        self._link_buffer.append_link(uri, filename)
        self._link_buffer.append_markup('", line %s in ' % line_no)
        self._link_buffer.append_markup(function_name, 'function-name')
        self._link_buffer.append_markup('\n')
        if text:
            self._link_buffer.append_markup('    %s\n' % text, 'text')

    def _append_footer(self, error, message):
        self._link_buffer.append_markup(error, 'error-class')
        self._link_buffer.append_markup(': ')
        self._link_buffer.append_markup(message, 'error-message')
        self._link_buffer.append_markup('\n\n')

    def _append_error(self, test, error):
        self._append_test_id(test.id())
        error = self._error_to_string(error)
        self._append_header(get_header(error))
        for stack in get_stacks(error):
            filename, line_no, function_name, text = parse_stack(stack)
            self._append_stack(filename, line_no, function_name, text)
        error, message = parse_footer(get_footer(error))
        self._append_footer(error, message)
        self._view.scroll_to_mark(self._view.get_buffer().get_insert(), 0)

    def _error_to_string(self, error):
        exc_type, value, tb = error
        tb = ''.join(traceback.format_exception(exc_type, value, tb))
        return tb.strip()

    def clear(self):
        self._link_buffer.clear()

    def startTest(self, test):
        pass

    def stopTest(self, test):
        pass

    def addSuccess(self, test):
        pass

    def done(self):
        pass

    def addError(self, test, error):
        self._append_error(test, error)

    def addFailure(self, test, failure):
        self._append_error(test, failure)


def get_header(traceback_string):
    return traceback_string.splitlines()[0]


def get_footer(traceback_string):
    return traceback_string.splitlines()[-1]


def parse_footer(footer):
    parsed = tuple(footer.split(': ', 1))
    if len(parsed) == 1:
        return parsed[0], ''
    return parsed


def get_stacks(traceback_string):
    lines = traceback_string.splitlines()[1:-1]
    stacks = []
    i = 0
    while i < len(lines):
        first, second = lines[i], lines[i+1]
        if second.lstrip().startswith('File "'):
            # Then the source line for 'first' is missing.
            second = ''
            i += 1
        else:
            i += 2
        stacks.append((first, second))
    return stacks


def parse_stack(stack):
    location_re = re.compile('File "([^"]+)", line (\d+), in (.+)')
    text = stack[1].strip()
    location = stack[0].strip()
    match = location_re.match(location)
    return match.group(1), int(match.group(2)), match.group(3), text
