import gtk
import sys

from tribunal.results import (
    get_header,
    get_footer,
    get_stacks,
    parse_footer,
    parse_stack,
    Results,
    )
from tribunal.testing import TestCase


class TestResults(TestCase):
    """Tests for the result-displaying widget."""

    def make_results(self):
        text_view = gtk.TextView()
        return Results(text_view)

    def setUp(self):
        self.results = self.make_results()
        self.buffer = self.results._link_buffer._buffer

    def get_buffer_text(self):
        return self.buffer.get_text(
            self.buffer.get_start_iter(), self.buffer.get_end_iter())

    def make_error(self, exc_factory):
        try:
            raise exc_factory()
        except:
            return sys.exc_info()

    def test_initially_empty(self):
        # The results view is empty on construction.
        self.assertEqual('', self.get_buffer_text())

    def test_add_success_changes_nothing(self):
        # The results view displays nothing when a success is reported.
        self.results.addSuccess(self)
        self.assertEqual('', self.get_buffer_text())

    def test_add_error_adds_message(self):
        # The results view displays any errors that are reported.
        error = self.make_error(RuntimeError)
        self.results.addError(self, error)
        text = self.get_buffer_text()
        self.assertIn('RuntimeError', text)
        self.assertIn(self.id(), text)

    def test_add_failure_adds_message(self):
        # The results view displays any failures that are reported.
        error = self.make_error(AssertionError)
        self.results.addFailure(self, error)
        text = self.get_buffer_text()
        self.assertIn('AssertionError', text)
        self.assertIn(self.id(), text)

    def test_clear_empties_all_messages(self):
        # Calling `clear` on the results view clears all messages.
        error = self.make_error(AssertionError)
        self.results.addFailure(self, error)
        self.results.clear()
        self.assertEqual('', self.get_buffer_text())


class TestTracebackParsing(TestCase):
    """Tests for traceback parsing.

    For any error, subunit gives us only the string. If we want tracebacks to
    look sexy, we need to parse them from their string form.
    """

    sample_traceback = """
Prefix: Traceback (most recent call last):
  File "foo.py", line 123, in crazy
    some_method()
  File "missing.pyc", line 44, in where_on_earth
  File "bar.py", line 235, in zany
    something_else()
exceptions.ValueError: message of doom
""".lstrip()

    def test_get_header(self):
        header = get_header(self.sample_traceback)
        self.assertEqual("Prefix: Traceback (most recent call last):", header)

    def test_get_footer(self):
        footer = get_footer(self.sample_traceback)
        self.assertEqual("exceptions.ValueError: message of doom", footer)

    def test_get_stacks(self):
        stacks = get_stacks(self.sample_traceback)
        self.assertEqual(
            [('  File "foo.py", line 123, in crazy', '    some_method()'),
             ('  File "missing.pyc", line 44, in where_on_earth', ''),
             ('  File "bar.py", line 235, in zany', '    something_else()')],
            stacks)

    def test_parse_stack(self):
        stack = ('  File "foo.py", line 123, in crazy', '    some_method()')
        parsed = parse_stack(stack)
        self.assertEqual(('foo.py', 123, 'crazy', 'some_method()'), parsed)

    def test_parse_footer(self):
        footer = "exceptions.ValueError: message of doom"
        self.assertEqual(
            ('exceptions.ValueError', 'message of doom'),
            parse_footer(footer))
        footer = "exceptions.ValueError: Note: message"
        self.assertEqual(
            ('exceptions.ValueError', 'Note: message'),
            parse_footer(footer))
