# Copyright 2008 Jonathan Lange
# Copyright 2010 Martin Pool

__all__ = [
    'TreeSuite',
    ]

import gtk
from pprint import pformat


class TribunalTestResult(object):
    """A TestResult, fed test outcomes by the TestRunner.

    This one retains some data itself, and puts other parts into a TreeStore
    that drives a tree display.
    
    This only deals with the data store, not at all with the view.
    
    The GtkTreeStore has two columns: the test id, and the outcome.

    This is a partially abstract base class; it needs some extra definitions
    of how the tests may into rows.

    :ivar store: gtk TreeStore
    """

    UNKNOWN = 'unknown'
    PASSED = 'passed'
    FAILED = 'failed'
    SKIPPED = 'skipped'
    XFAIL = 'xfail'

    # COL_DETAIL_TEXT = 2 # no longer used
    COL_OUTCOME = 1

    def __init__(self):
        self.store = gtk.TreeStore(str, str, str)
        self._test_id_to_path = {}
        self._path_to_test_id = {}
        self._details_by_id = {}
        self._error_by_id = {}

    def _clear(self):
        """Completely reset all information in the TreeSuite object."""
        self.store.clear()
        self._test_id_to_path.clear()
        self._path_to_test_id.clear()
        self._details_by_id.clear()
        self._error_by_id.clear()

    # from TestResult
    def startTest(self, test):
        self.add_test_id(test.id())

    def add_test_id(self, test_id):
        """Must be implemented to add the row for the test in the desired place."""
        raise NotImplementedError(self.add_test_id)

    # from TestResult
    def addError(self, test, error, details=None):
        self._store_result(test, self.FAILED, error, details)

    # from TestResult
    def addExpectedFailure(self, test, error=None, details=None):
        self._store_result(test, self.XFAIL, error, details)

    # from TestResult
    def addFailure(self, test, error=None, details=None):
        self._store_result(test, self.FAILED, error, details)

    # from TestResult
    def addSkip(self, test, reason=None, details=None):
        self._store_result(test, self.SKIPPED, None, details, skip_reason=reason)

    # from TestResult
    def addSuccess(self, test, details=None):
        self._store_result(test, self.PASSED, None, details)

    def _store_result(self, test, outcome, exc_info, details,
        skip_reason=None):
        """Store a result passed from the test.

        :param outcome: One of the outcome constants, eg SKIPPED
        :param err: Either an exception or a string (in the case of Skip) 
            or None
        :param details: subunit details dictionary or none
        :param test: A test object on which you can at least call .id().
        """
        test_id = test.id()
        self.store.set_value(
            self.store.get_iter(self.get_tree_path(test_id)),
            self.COL_OUTCOME,
            outcome)
        self._details_by_id[test_id] = details
        if exc_info:
            self._error_by_id[test_id] = '%s: %s' % exc_info[0:2]
        else:
            self._error_by_id[test_id] = skip_reason

    # from TestResult
    def stopTest(self, test):
        pass

    def get_detail_text_by_test_id(self, test_id):
        return self._details_by_id[test_id]

    def get_outcome_by_test_id(self, test_id):
        return self.store.get(
            self.store.get_iter(self.get_tree_path(test_id)),
            self.COL_OUTCOME)[0]

    def get_error_by_test_id(self, test_id):
        """Returns a single (probably short) string describing the error 
        or skip."""
        return self._error_by_id[test_id]

    def get_test_id(self, tree_path):
        """Return the fully-qualified test name.

        :param tree_iter: A GTK tree path.
        :return: The fully-qualified Python name of the object, or None if
            there is no test at that path.
        """
        return self._path_to_test_id.get(tree_path, None)

    def get_tree_path(self, test_id):
        return self._test_id_to_path[test_id]

    def _iterate(self, iter=None):
        """Iterate through the immediate children of C{iter} (including
        C{iter} itself.

        :param iter: A L{gtk.TreeIter}. If C{None}, defaults to the root node.
        :return: Yields L{gtk.TreeIter} objects immediately under C{iter}.
        """
        if iter is None:
            iter = self.store.get_iter_root()
        while iter is not None:
            yield iter
            iter = self.store.iter_next(iter)


class TableTestResults(TribunalTestResult):
    """Show test results in a table with no nesting, with these columns:

    * outcome (pass/fail/etc)
    * test id
    * elapsed time
    * why
    """

    COL_TEST_ID = 0
    COL_WHY = 2

    def add_test_id(self, test_id):
        new_iter = self.store.append(None, (test_id, self.UNKNOWN, ''))
        path = self.store.get_path(new_iter)
        self._test_id_to_path[test_id] = path
        self._path_to_test_id[path] = test_id


class TreeSuite(TribunalTestResult):
    """Combination model / view that represents the tests that the UI has.
    
    This fills a TreeStore which has 3 columns:
     0: the test id
     1: passed/failed/unknown
     2: other details about the test
    """

    def __init__(self, view):
        TribunalTestResult.__init__(self)
        self.view = view
        view.set_model(self.store)
        icon_cell = gtk.CellRendererPixbuf()
        test_name_cell = gtk.CellRendererText()
        test_name_col = gtk.TreeViewColumn()
        test_name_col.pack_start(icon_cell, expand=False)
        test_name_col.pack_start(test_name_cell, expand=True)
        test_name_col.add_attribute(test_name_cell, 'text', 0)
        test_name_col.set_cell_data_func(icon_cell, self.update_test_result)
        view.append_column(test_name_col)
        self._autoscroll_on_add = True

    def update_test_result(self, column, cell, model, iterator):
        """Update the tree to show a success or fail icon.

        Called when the underlying model is changed.
        """
        test_result = model.get_value(iterator, self.COL_OUTCOME)
        if test_result == self.UNKNOWN:
            cell.set_property('visible', False)
            cell.set_property('stock-id', None)
        elif test_result == self.PASSED:
            cell.set_property('visible', True)
            cell.set_property('stock-id', gtk.STOCK_YES)
        elif test_result == self.FAILED:
            cell.set_property('visible', True)
            cell.set_property('stock-id', gtk.STOCK_NO)
        else:
            cell.set_property('visible', True)
            cell.set_property('stock-id', gtk.STOCK_INFO)

    def add_test_id(self, test_id):
        tree_iter = self._add_path_to_tree_store(test_id.rsplit('.', 2))
        path = self.store.get_path(tree_iter)
        self._test_id_to_path[test_id] = path
        self._path_to_test_id[path] = test_id
        self.view.expand_to_path(path)
        if self._autoscroll_on_add:
            self.view.scroll_to_cell(path)

    def _find(self, parent, name):
        """Look for an object called C{name} immediately under C{parent}.

        :param parent: The L{gtk.TreeIter} to search under. C{None} implies
            the root of the tree.
        :param name: The name of the object.
        :return: The L{gtk.TreeIter} pointer to the object if found, C{None}
            otherwise.
        """
        if parent is not None:
            parent = self.store.iter_children(parent)
        for i in self._iterate(parent):
            if self.store.get_value(i, 0) == name:
                return i
        return None

    def _add_path_to_tree_store(self, pathSegments, parent=None):
        """Add the path represented by C{pathSegments} to the tree store.

        :param pathSegments: A list of path segments.
        :param parent: The L{gtk.TreeIter} to add them under. Defaults to the
            root of the tree.
        :return: The L{gtk.TreeIter} of the newly-added node.
        """
        if len(pathSegments) == 0:
            return parent
        head, tail = pathSegments[0], pathSegments[1:]
        i = self._find(parent, head)
        if i is None:
            i = self.store.append(parent, (head, self.UNKNOWN, ''))
        return self._add_path_to_tree_store(tail, i)

